#include <avr\io.h>
#include <avr\interrupt.h>
#include <util\delay.h>
#include <string.h>
#include <stdio.h>

#include "macro.h"
#include "led_driver.h"
#include "spi.h"
#include "uart.h"

char buffer[7];
char bufferInteger[6];
char bufferDecimal[6];
char indicatorDigits[7]; // Five digits incl comma, plus alarm 

volatile unsigned int brightness; // 0 - 100
volatile unsigned int brightnessTimer, alarmTimer;
volatile bool alarm, alarmState;

void led_driver_init(void){
	brightness = 0;
	brightnessTimer = 0;
	alarmTimer = 0;
	SET_BIT(PORTB, OE);
	alarm = ALARMOFF;
	alarmState = FALSE;
}

// Convert from double to separate digits
void led_driver_update(int integer, int decimal){
	int i, j;
	int activeDigits;
	
	// Be sure the buffers are empty
	//printf("2");
	i=0;
	for (i=0; i<7; i++){
		buffer[i] = ' ';
		}
	
	for (i=0; i<5; i++){
		bufferInteger[i] = ' ';
		bufferDecimal[i] = ' ';
	}
	
	// Convert the integer to chars 
	sprintf(bufferInteger, "%d", integer);
	// Number of active digits
	activeDigits = strlen(bufferInteger);


	i=0;
	// Add integer chars to buffer
	while (bufferInteger[i] != '\0'){
		buffer[i] = bufferInteger[i];
		i++;
	}
	
	// If any decimals, convert them to chars and add comma to buffer
	// add to number of active digits
	sprintf(bufferDecimal, "%d", decimal);
	buffer[i] = '.';
	activeDigits += strlen(bufferDecimal);
	

	// Add decimal chars to buffer
	j=0;
	while (bufferDecimal[j] != '\0'){
		buffer[i+1] = bufferDecimal[j];
		j++;
		i++;
	}
	buffer[6] = '\0';

	
	// Set alarm
	if(alarm) indicatorDigits[5] = ALARM;
	else indicatorDigits[5] = OFF;
	

	// Turn off non-active digits
	for (i=0; i<(5-activeDigits); i++){
		indicatorDigits[i] = OFF;
	}

	// Convert the char buffer to numeric seven-segment values
	for (j=0;j<6; j++){
		if (buffer[j] == '0') indicatorDigits[i] = ZERO;
		else if (buffer[j] == '1') indicatorDigits[i] = ONE;
		else if (buffer[j] == '2') indicatorDigits[i] = TWO;
		else if (buffer[j] == '3') indicatorDigits[i] = THREE;
		else if (buffer[j] == '4') indicatorDigits[i] = FOUR;
		else if (buffer[j] == '5') indicatorDigits[i] = FIVE;
		else if (buffer[j] == '6') indicatorDigits[i] = SIX;
		else if (buffer[j] == '7') indicatorDigits[i] = SEVEN;
		else if (buffer[j] == '8') indicatorDigits[i] = EIGHT;
		else if (buffer[j] == '9') indicatorDigits[i] = NINE;
		else if (buffer[j] == '.') {
			i--;
			indicatorDigits[i] = (indicatorDigits[i]+COMMA);
		}
		else indicatorDigits[i] = OFF;
		i++;
	}

	// Tramsmit the digits over SPI to the LED drivers
	for (i=0; i<6; i++){
		SPI_MasterTransmit(indicatorDigits[i]);
	}
	
	// Latch enable 
	SET_BIT(PORTB, SS0);
	SET_BIT(PORTB, SS1);
	SET_BIT(PORTB, SS2);

	_DELAY_US(1);

	// Latch enable off
	CLEAR_BIT(PORTB, SS0);
	CLEAR_BIT(PORTB, SS1);
	CLEAR_BIT(PORTB, SS2);
		
}

// Not used
/*
void led_driver_transmitDigits(){
	cli();
	int i;
	for (i=0; i<6; i++){
		SPI_MasterTransmit(indicatorDigits[i]);
	}
	sei();
}
*/
 
// Called from main-loop for timing of the brightness function
void led_driver_updateBrightness(){
	cli();
	if(brightness == 1) brightness = 2;
	if(brightnessTimer == 1 && brightness != 0){ 
		CLEAR_BIT(PORTB, OE); // Output enable
	}
	else if ((brightnessTimer == brightness || brightness == 0) && brightness != 100){
		SET_BIT(PORTB, OE); // Output enable off
		}
	if(brightnessTimer == 100) brightnessTimer = 0;
	brightnessTimer++;	
	sei();
}

// Called from main-loop for timing of the alarm function
// Flashing the alarm sign when activated
void led_driver_updateAlarm(){
	if(alarmTimer == 1 && alarmState == TRUE) alarm = ALARMON;
	else if (alarmTimer == 50) alarm = ALARMOFF;
	if (alarmTimer == 80) alarmTimer = 0;
	if (alarmState == FALSE && alarmTimer == 1){
		alarmTimer = 0;
		alarm = ALARMOFF;
	}
	alarmTimer++;
	//printf("%d", alarm);  
}

// To set the brightness level 0 - 100 %
void led_driver_setBrightness(int value){
	brightness = value;
}

// To set the alarm from other modules
void led_driver_setAlarm(bool state){
	alarmState = state;
}
