#include <avr\io.h>
#include <avr\interrupt.h>
#include <util\delay.h>
#include <stdio.h>
#include <stdlib.h>

#include "status.h"
#include "adc.h"
#include "macro.h"
#include "spi.h"
#include "uart.h"
#include "status.h"
#include "led_driver.h"
#include "timer.h"

volatile unsigned int mainTimer;
volatile bool alarmState;


int main(void){

	// Initializing
	uart_init(15); // baud rate 19200
	led_driver_init();
	SPI_MasterInit();
	adc_init();
	status_init();
	timer_init();
	sei();
	printf("Running\n\r");
	
	mainTimer = 0;
	alarmState = FALSE;

	// Initial value of the display
	led_driver_update(0,0);

	while(TRUE){
		
		// Ensures 50 Hz running
		if(timer_getTime1() && (abs(timer_getFastTimer()-mainTimer)>=300)){
			mainTimer = timer_getFastTimer();	
			
			if(TEST_BIT(PINC, DEMO1) && !TEST_BIT(PINC, DEMO2)){
				// Brightness adjust
				led_driver_setBrightness(adc_getValue()/20);
				
				// RS-232
				uart_timer();
			}
			
			// Demo mode
			else if (!TEST_BIT(PINC, DEMO1) && TEST_BIT(PINC, DEMO2)){
				// Set low-pass filtered ADC value at display
				led_driver_update(adc_getValue(),0);

				// Flash alarm if value exceeds 1900
				if(adc_getValue() >= 1900) led_driver_setAlarm(TRUE);
				else led_driver_setAlarm(FALSE);
			}

			// Flash status LED
			status_flash();

			// Timing of the alarm sign
			led_driver_updateAlarm();
		}	
	}
	return 0;
}



/*Interrupt
 **********/

// ADC conversion complete
/*ISR(ADC_vect){
	if(adc_read() > (adc_read_temp+2) || adc_read() < (adc_read_temp-2)){
		adc_read_temp = adc_read();
	} 

	//printf("ADC: %d\n\r", adc_read_temp*2/20);
}
*/

// Timer0
ISR(TIMER0_OVF_vect){
	// Timer
	timer_increase();
	// Indicator brigthness
	led_driver_updateBrightness();
}

// Interrupts when receiving RS-232 data
ISR(USART0_RX_vect){
	uart_handleRx();
	uart_setTimer();
}

