#include <avr\io.h>
#include <avr\interrupt.h>
#include <util\delay.h>
#include <string.h>
#include <stdio.h>

#include "macro.h"
#include "spi.h"


void SPI_MasterInit(void){
	// Set MOSI and SCK output, all others input
	DDRB = (1<<MOSI)|(1<<SCK)|(1<<SS0)|(1<<OE);

	// Enable SPI, Master, set clock rate fck/128
	SPCR = (1<<SPE)|(1<<MSTR)|(1<<SPR0)|(1<<SPR1);

	// Enable interrupt
	EIMSK = (1<<INT4);
	EICRB |= (1<<ISC41);
	DDRE |= (0<<INT4);


//	printf("SPI initialized\n\r");
}


void SPI_MasterTransmit(char data){
	// Byte in register
	SPDR = data;
	// Wait for transmission complete
	while(!(SPSR & (1<<SPIF)));

}


void SPI_can_MasterTransmit(char cData){
	 // Put data in register
	SPDR = cData;

	// Wait for transmission complete
	while(!(SPSR & (1<<SPIF)));
}


char SPI_SlaveTransmit(void){
	// Slave select
	CLEAR_BIT(PORTB, SS0);

	// Put dummy data in register
	SPDR = DUMMY;

	// Wait for transmission complete
	while(!(SPSR & (1<<SPIF)));

	// Deselect slave
	SET_BIT(PORTB, SS0);

	// Return data from slave
	return SPDR;
}

void SPI_StringTransmit(char* string){
	if(strlen(string) < MAX_STRING_LENGTH){
		int i = 0;
		while(string[i] != '\0'){
			SPI_MasterTransmit(string[i]);
			i++;
		}
		//SPI_MasterTransmit('\0');
	}
}

