#include <avr\io.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <avr\interrupt.h>
#include "uart.h"
#include "macro.h"
#include "led_driver.h"

char rxTemp[8];
char rxNew[8];
char rxOld[8];
//char integers[6];
//char decimals[6];
int rxCounter;
int uartTimer;
int integers, dec;

void uart_init(unsigned int baud)
{
	// Set baud rate
	UBRR0H = (unsigned char)(baud>>8);
	UBRR0L = (unsigned char)baud;
	
	// Enable receiver and transmitter
	UCSR0B = (1<<RXEN0)|(1<<TXEN0);
	
	//Enable recieve interrupt
	UCSR0B |= (1<<RXCIE0);
	
	// Set frame format: 2stop bit, 8data, even parity
	UCSR0C = (1<<USBS0)|(1<<UCSZ00)|(1<<UCSZ01)|(1<<UPM01);
	
	rxCounter = 0;
	uartTimer = 10;
	integers = 0;
	dec = 0;
	fdevopen(uart_transmit, uart_receive);
	printf("Initializing\n\r");
}


void uart_transmit(unsigned char data)
	{
	// Wait for empty transmit buffer
	while (!(UCSR0A & (1<<UDRE0)));
	
	// Put data into buffer, sends the data
	UDR0 = data;
}


unsigned char uart_receive(void)
	{
	// Wait for data to be received
	while (!(UCSR0A & (1<<RXC0)));
	
	// Get and return received data from buffer
	
	return UDR0;
}

// Save received data
void uart_handleRx(){
	rxNew[rxCounter] = uart_receive();	
	rxCounter++;
}

// Convert received data to integers and send to led driver module
void uart_printRx(){
	// Run if the there is new data in the buffer
	if(strcmp(rxNew,rxOld)){
		rxCounter = 0;
		int commaPosition = 0;
		char decimals[6];
		int i = 0;
		int j = 0;
		
		// Take a copy to compare the next received data
		// Save comma position
		// Set alarm
		for (i=0;i<7;i++){
			rxOld[i] = rxNew[i];
			if(rxNew[i] == '.') commaPosition = i+1;
			else if(rxNew[i] == 'A') led_driver_setAlarm(TRUE);
			else if (rxNew[i] == 'a') led_driver_setAlarm(FALSE);
		}
	
		// Copy digits after comma
		if(commaPosition > 0){
			for (i=commaPosition;i<6;i++){
				decimals[j] = rxOld[i];
				j++;
			}
		}
		
		// Converts all digits before the comma, integers
		integers = atoi(rxOld);

		// Convert the decimals
		dec = atoi(decimals);
		
		// Flush buffers
		i = 0;
		while (rxOld[i]!='\0'){
			rxOld[i] = ' ';
			rxNew[i] = ' ';
			i++;
		}

		for (i=0;i<6;i++){
			decimals[i] = ' ';
		}

	}

	// Update display
	led_driver_update(integers, dec);
}


void uart_setTimer(){
	uartTimer = 5;
}


void uart_timer(){
	if(uartTimer == 0) 
	uart_printRx();
	if(uartTimer >= 1) uartTimer--;
}
